from logger_local.LoggerComponentEnum import LoggerComponentEnum

# TODO We should bring this value from entity_type_view
EVENT_ENTITY_TYPE_ID = 14

EVENT_LOCAL_PYTHON_COMPONENT_ID = 247
EVENT_LOCAL_PYTHON_COMPONENT_NAME = "event-local-restapi-python-serverless"
DEVELOPER_EMAIL = "akiva.s@circ.zone"

EVENTS_LOCAL_CODE_LOGGER_OBJECT = {
    "component_id": EVENT_LOCAL_PYTHON_COMPONENT_ID,
    "component_name": EVENT_LOCAL_PYTHON_COMPONENT_NAME,
    "component_category": LoggerComponentEnum.ComponentCategory.Code.value,
    "developer_email": DEVELOPER_EMAIL,
}



from src.handler import getEventsFromOpensearch
from tests.opensearch_queries import (
    opensearch_query1, opensearch_query2, opensearch_query3, opensearch_query4, opensearch_query5,
    opensearch_query6, opensearch_query7, opensearch_query8, opensearch_query9, opensearch_query10,
)

queries = [
    opensearch_query1, opensearch_query2, opensearch_query3, opensearch_query4, opensearch_query5,
    opensearch_query6, opensearch_query7, opensearch_query8, opensearch_query9, opensearch_query10,
]


def test_get_events_from_opensearch():
    request_parameters = {
        "langCode": "en",
        "limit": 10,
        "isTestData": True,
    }

    responses = []

    for opensearch_query in queries:

        response = getEventsFromOpensearch(request_parameters, opensearch_query['query'])

        assert response is not None
        assert response["_shards"]["failed"] == 0
        assert response["_shards"]["successful"] > 0
        assert response["_shards"]["total"] == response["_shards"]["successful"] + response["_shards"]["failed"]

        responses.append(response)

    # print(response)

    assert responses[0]["_shards"]["failed"] == 0
    assert responses[0]["_shards"]["successful"] > 0
    assert responses[0]["_shards"]["total"] == response["_shards"]["successful"] + response["_shards"]["failed"]


if __name__ == "__main__":
    test_get_events_from_opensearch()



from logger_local.LoggerLocal import Logger
from python_sdk_local.http_response import handler_decorator
from python_sdk_local.utilities import DEFAULT_LANG_CODE_STR

from opensearch_local.our_opensearch import OurOpenSearch

from .constants_event_local import EVENTS_LOCAL_CODE_LOGGER_OBJECT
from .events_local import EventsLocal

logger = Logger.create_logger(object=EVENTS_LOCAL_CODE_LOGGER_OBJECT)


@handler_decorator(logger)
def createHandler(request_parameters: dict) -> dict:
    # In all serverless-com handlers' logger.start()- Add optional parameter
    # to logger.start() which called api_call, so logger.start will call
    # api_management to insert into api_call_table all fields including
    # session_id.
    """Read parameters from the payload and insert them"""
    print("in createHandler request_parameters", request_parameters)
    is_test_data = request_parameters.get("isTestData", False)
    event_id = EventsLocal(is_test_data=is_test_data).insert_event_data(
        request_parameters
    )
    message = {"message": "Event created successfully", "event_id": event_id}
    logger.info(
        object={
            "message": message,
            "request_parameters": request_parameters,
        }
    )
    return message


# TODO: except ValueError as exception, BAD_REQUEST / INTERNAL_SERVER_ERROR...


@handler_decorator(logger)
def deleteHandler(request_parameters: dict) -> dict:
    """url/5   (5 is the eventId)"""

    event_id = request_parameters.get("eventId")
    is_test_data = request_parameters.get("isTestData", False)
    EventsLocal(is_test_data=is_test_data).delete_by_event_id(event_id=event_id)
    message = {"message": "Event deleted successfully"}
    return message


# TODO This should be imported from entity-type-python-package (this file will be generated by Sql2Code)
@handler_decorator(logger)
def getAllHandler(request_parameters: dict) -> list:
    """url?langCode=en&limit=10"""

    lang_code_str = request_parameters.get("langCode") or DEFAULT_LANG_CODE_STR
    limit = request_parameters.get("limit")
    is_test_data = request_parameters.get("isTestData", False)
    events_list = EventsLocal(is_test_data=is_test_data).select_all_events(
        lang_code_str, limit=limit
    )
    return events_list


@handler_decorator(logger)
def getEventByProfileIdHandler(request_parameters: dict) -> list:
    """url/5?limit=10   (5 is the profileId)"""

    request_parameters = request_parameters
    profile_id = request_parameters.get("profileId")
    limit = request_parameters.get("limit")

    is_test_data = request_parameters.get("isTestData", False)
    events_list = EventsLocal(is_test_data=is_test_data).select_events_by_profile_id(
        profile_id, limit=limit
    )
    return events_list


@handler_decorator(logger)
def getEventByEventIdHandler(request_parameters: dict) -> dict:
    """url/5?isTestData=true   (eventId)"""

    event_id = request_parameters.get("eventId")

    is_test_data = request_parameters.get("isTestData", False)
    event = EventsLocal(is_test_data=is_test_data).select_by_event_id(event_id)
    if not event:
        raise Exception(f"Event not found with event_id: {event_id}")
    return event


@handler_decorator(logger)
def updateHandler(request_parameters: dict) -> dict:
    """url/5  (eventId)
    Read parameters from the payload and update them"""

    event_id = request_parameters.get("eventId")
    is_test_data = request_parameters.get("isTestData", False)
    if not event_id:
        raise Exception("event_id is required")

    EventsLocal(is_test_data=is_test_data).update_by_event_id(
        event_id, request_parameters
    )
    message = {"message": "Event updated successfully"}
    return message


@handler_decorator(logger)
def getEventByEventTitleHandler(request_parameters: dict) -> list:  # TODO: test
    """url/SomeTitle?limit=10"""

    title = request_parameters.get("title")

    limit = request_parameters.get("limit")
    is_test_data = request_parameters.get("isTestData", False)
    events_list = EventsLocal(is_test_data=is_test_data).select_events_by_title(
        title, limit=limit
    )
    return events_list


def getEventsFromOpensearch(request_parameters: dict, opensearch_query: dict) -> list:
    """url?langCode=en&limit=10"""
    our_opensearch = OurOpenSearch(buffer_size=5)
    lang_code_str = request_parameters.get("langCode") or DEFAULT_LANG_CODE_STR

    limit = request_parameters.get("limit")
    is_test_data = request_parameters.get("isTestData", False)

    bool_query = {
        "bool": {
            "must": [
                opensearch_query,  # Keep the existing query conditions
                {"match": {"lang_code": lang_code_str}},  # Add lang_code filter
                {"match": {"is_test_data": is_test_data}},  # Add is_test_data filter
            ]
        }
    }

    new_query = {
        "query": bool_query,
        "size": limit
    }

    events_list = our_opensearch.search(
        index="event",
        body=new_query
    )
    return events_list



from http import HTTPStatus

from python_sdk_remote.http_response import create_authorization_http_headers
from user_context_remote.user_context import UserContext

from src.events_local import EventsLocal
from src.handler import deleteHandler

user_jwt = UserContext().get_user_jwt()


class TestDelete:
    def test_valid_id(self):
        event = {
            "pathParameters": {"eventId": EventsLocal().get_test_event_id()},
            "headers": create_authorization_http_headers(user_jwt),
        }
        response = deleteHandler(event, None)
        assert response["statusCode"] == HTTPStatus.OK.value
        assert response["body"] == '{"message": "Event deleted successfully"}'

        EventsLocal().update_by_column_and_value(
            column_name="event_id",
            column_value=event["pathParameters"]["eventId"],
            data_dict={"end_timestamp": None},
        )

    def test_invalid_id(self):
        event = {
            "pathParameters": {"eventId": "invalid"},
            "headers": create_authorization_http_headers(user_jwt),
        }
        response = deleteHandler(event, None)
        assert response["statusCode"] != HTTPStatus.OK.value
        # Add event_id number
        assert '{"error": ' in response["body"]


if __name__ == "__main__":
    test = TestDelete()
    test.test_valid_id()
    # test.test_invalid_id()



# TODO Change the directory structure (i.e. this move this file to event-main-local-restapi-python-package-serverless-com directory)
import setuptools

# TODO: Change the PACKAGE_NAME to the name of the package - Either xxx-local or xxx-remote (without the -python-package suffix).
# TODO: Only lowercase, no underlines.
# Used by pypa/gh-action-pypi-publish
# Package Name should be identical to the inner directory name
# Changing the package name here, will cause changing the package directory name as well
# PACKAGE_NAME should be singular if handling only one instance
# PACKAGE_NAME should not include the word "main"
PACKAGE_NAME = "event-local"  # e.g.: queue-local, without python-package suffix

package_dir = PACKAGE_NAME.replace("-", "_")
# If we need backward-compatible:
# old_package_dir = "old_package_name"

setuptools.setup(
    name=PACKAGE_NAME,  # https://pypi.org/project/event-local
    version="0.0.18",  # increase this number every time you make a change you want to publish. After 0.0.9 switch to 0.0.10 and not 0.1.0
    author="Circles",
    author_email="info@circlez.ai",
    description=f"PyPI Package for Circles {PACKAGE_NAME} Python",
    long_description=f"PyPI Package for Circles {PACKAGE_NAME} Python",
    long_description_content_type="text/markdown",
    url=f"https://github.com/circles-zone/{PACKAGE_NAME}-python-package",
    packages=[package_dir],
    # packages=[package_dir, old_package_dir],
    # package_dir={package_dir: f'{package_dir}/src'},
    # TODO Unfortunetly in event-main-local-restapi there are not repo-directory and no package directory
    package_dir={package_dir: "src"},
    # package_dir={package_dir: f'{package_dir}/src', old_package_dir: f'{package_dir}/src'},
    package_data={package_dir: ["*.py"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
    ],
    # TODO: Update which packages to include with this package in production (dependencies) - Not for development/testing
    install_requires=[
        "logger-local",  # TODO: in -remote package please use logger-remote instead.
        "database-mysql-local",  # TODO: In -remote package please delete this line.
        "python-sdk-remote",
        "opensearch-local >= 0.0.8"
    ],
)



# TODO Can we use event-local package instead of this file

# TOOD Can we delete GenericCRUD?
from database_mysql_local.generic_crud import DEFAULT_SQL_SELECT_LIMIT  # , GenericCRUD
from database_mysql_local.generic_crud_ml import GenericCRUDML
from location_local.locations_local_crud import LocationsLocal
from logger_local.MetaLogger import MetaLogger
from .constants_event_local import EVENTS_LOCAL_CODE_LOGGER_OBJECT, EVENT_ENTITY_TYPE_ID
from .validate_inputs import ValidateInputs
from language_remote import lang_code
from dotenv import load_dotenv

load_dotenv()  # take environment variables from .env.


# OLD class EventsLocal(GenericCRUD, metaclass=MetaLogger, object=EVENTS_LOCAL_CODE_LOGGER_OBJECT):
class EventsLocal(
    GenericCRUDML, metaclass=MetaLogger, object=EVENTS_LOCAL_CODE_LOGGER_OBJECT
):

    def __init__(self, is_test_data: bool = False) -> None:
        super().__init__(
            default_schema_name="event",
            default_column_name="event_id",
            default_table_name="event_table",
            default_view_table_name="event_view",
            default_view_with_deleted_and_test_data="event_general_with_deleted_and_test_data_view",
            is_test_data=is_test_data,
        )
        self.validate_inputs = ValidateInputs(is_test_data=is_test_data)

        # TODO: use dependency injection in future versions
        # vector_db = []
        # vector_db[0] = OpenSearchIngester(is_test_data=is_test_data)
        # vector_db[1] = PGvector(is_test_data=is_test_data)
        # moved to genereic_crud

    # TODO: test
    def select_all_events(
        self, lang_code_str: lang_code.LangCode, limit: int = None
    ) -> list:
        """Returns all the rows in the event_table"""
        limit = limit or 10000
        # TODO If we don't have result int the profile preferred language, we should bring the English version
        # TODO If event has multiple names we should bring the one which is_main = TRUE
        sql_query = f"""
            SELECT event.event_id         AS 'event_id',
                   {EVENT_ENTITY_TYPE_ID} AS 'item_type_id',
                   event.location_id,
                   event.organizers_profile_id,
                   event.website_url,
                   event.facebook_event_url,
                   event.meetup_event_url,
                   event_ml.title
            FROM event.event_view AS event
                     LEFT JOIN event.event_ml_view AS event_ml ON event.event_id = event_ml.event_id
                AND event_ml.lang_code = %s
            LIMIT {limit}"""
        self.cursor.execute(sql_query, (lang_code_str.value,))
        columns = (
            "event_id, item_type_id, location_id, organizers_profile_id, website_url, "
            "facebook_event_url, meetup_event_url, title"
        )
        events_dict = self.convert_multi_to_dict(self.cursor.fetchall(), columns)
        return events_dict

    def select_events_by_title(self, title: str, limit: int = None) -> list:
        """Returns all the events that have the given title"""
        limit = limit or DEFAULT_SQL_SELECT_LIMIT
        events_list = self.select_multi_dict_by_column_and_value(
            view_table_name="event_ml_view",
            column_name="title",
            column_value=title,
            limit=limit,
        )
        return events_list

    # TODO: test
    def select_events_by_profile_id(self, profile_id: int, limit: int = None) -> list:
        """Returns all the events that are related to the given profile_id"""
        self.validate_inputs.validate_id(profile_id, schema="profile")
        limit = limit or DEFAULT_SQL_SELECT_LIMIT
        # TODO: should we move this to a view?
        sql_query = f"""
        WITH EventCommonGroups AS (
            SELECT event.event_id,
                   event_ml.title,
                   {EVENT_ENTITY_TYPE_ID}        AS item_type,
                   COUNT(group_profile.group_id) AS common_group_count
            FROM event.event_view AS event
                     LEFT JOIN
                 event.event_ml_view AS event_ml ON event.event_id = event_ml.event_id
                     LEFT JOIN
                 event_group.event_group_view AS event_group ON event.event_id = event_group.event_id
                     LEFT JOIN
                 group_profile.group_profile_view AS group_profile
                 ON event_group.group_id = group_profile.group_id AND
                    group_profile.profile_id = %s
            GROUP BY event.event_id, event_ml.title
        )

        SELECT event.event_id,
               event.location_id,
               event.organizers_profile_id,
               event.website_url,
               event.facebook_event_url,
               event.meetup_event_url,
               event.registration_url,
               EventCommonGroups.title,
               EventCommonGroups.item_type,
               EventCommonGroups.common_group_count,
               event_location.location_id AS event_location_id,
               (COALESCE(1 / ST_Distance(
                       event_location.coordinate, COALESCE(location_profile.coordinate, event_location.coordinate)), 0)) AS distance_rank,
               (COALESCE(1 / ST_Distance(
                       event_location.coordinate, COALESCE(location_profile.coordinate, event_location.coordinate)), 0)) +
                        COALESCE(EventCommonGroups.common_group_count / 5, 0)               AS final_rank
        FROM EventCommonGroups
                 LEFT JOIN
             event.event_view AS event ON EventCommonGroups.event_id = event.event_id
                 LEFT JOIN
             (SELECT event_location.event_id,
                     event_location.location_id,
                     location.coordinate
              FROM event_location.event_location_view AS event_location
                       LEFT JOIN location.location_view AS location ON event_location.location_id = location.location_id) AS event_location
             ON event.event_id = event_location.event_id
                 LEFT JOIN
             (SELECT location_profile.location_id,
                     location.coordinate,
                     location_profile.profile_id
              FROM location_profile.location_profile_view AS location_profile
                       LEFT JOIN location.location_view AS location ON location_profile.location_id = location.location_id
              ORDER BY location_profile.start_timestamp DESC
              LIMIT 1) AS location_profile ON location_profile.profile_id = %s
                 LEFT JOIN
             location.location_view AS location ON location.location_id = location_profile.location_id
        ORDER BY final_rank DESC
        LIMIT {limit};
        """

        self.cursor.execute(sql_query, (profile_id, profile_id))
        columns = (
            "event.event_id, eventlocation_id, eventorganizers_profile_id, eventwebsite_url, "
            "eventfacebook_event_url, eventmeetup_event_url, eventregistration_url, title, item_type, "
            "common_group_count, event_location_id, distance_rank, final_rank"
        )
        events_dict = self.convert_multi_to_dict(self.cursor.fetchall(), columns)

        return events_dict

    def select_by_event_id(self, event_id: int) -> dict:
        """Returns the event row with the given event_id"""
        self.validate_inputs.validate_id(event_id)
        # TODO add support to event-external
        # select_clause = "event_id, location_id, organizers_profile_id, website_url, facebook_event_url, meetup_event_url, registration_url"
        select_clause = "*"
        event = self.select_one_dict_by_column_and_value(
            column_value=event_id, select_clause_value=select_clause
        )
        return event

    def insert_event_data(self, event_data: dict) -> int:
        """Inserts a row into the event_table and returns the id of the inserted row"""
        self.validate_inputs.validate_input_dict(input_dict=event_data)
        event_id = self.insert(data_dict=event_data)
        # for v_db in self.vector_db:
        #     v_db.insert(data_dict=event_data)
        return event_id

    def update_by_event_id(self, event_id: int, event_data: dict) -> int:
        """Updates the event row with the given event_id"""
        self.validate_inputs.validate_id(event_id)
        self.validate_inputs.validate_input_dict(input_dict=event_data)
        amount_of_events_updated = self.update_by_column_and_value(
            column_value=event_id, data_dict=event_data
        )
        return amount_of_events_updated

    def delete_by_event_id(self, event_id: int) -> int:
        """Deletes the event row with the given event_id"""
        self.validate_inputs.validate_id(event_id)
        amount_of_events_deleted = self.delete_by_column_and_value(
            column_value=event_id
        )
        return amount_of_events_deleted

    def get_test_event_id(self) -> int:
        location_id = LocationsLocal().get_test_location_id()
        test_event_id = super().get_test_entity_id(
            schema_name="event",
            view_name="event_view",
            entity_name="event",
            insert_function=self.insert,
            insert_kwargs={"location_id": location_id},
        )
        return test_event_id

    # Utilities
    def merge_event_ml_entities(
        self, entity_id1: int, entity_id2: int, main_entity_ml_id: int
    ):
        super().merge_ml_entities(
            entity_id1=entity_id1,
            entity_id2=entity_id2,
            main_entity_ml_id=main_entity_ml_id,
        )

    def merge_event_entities(
        self,
        entity_id1: int,
        entity_id2: int,
    ):
        super().merge_entities(entity_id1=entity_id1, entity_id2=entity_id2)

    # Metrics

    # Used by platform-metrics
    # TODO Should be used by event-metrics
    def get_number_of_events(self) -> int:
        sql_statement = "SELECT COUNT(*) FROM event.event_table"
        self.cursor.execute(sql_statement=sql_statement)
        number_of_events = self.cursor.fetchone()[0]
        return number_of_events



import json
from http import HTTPStatus

from python_sdk_remote.http_response import create_authorization_http_headers
from user_context_remote.user_context import UserContext

from src.events_local import EventsLocal
from src.handler import updateHandler

user_jwt = UserContext().get_user_jwt()


class TestUpdate:
    @staticmethod
    def test_update_valid_input():
        event = {
            "pathParameters": {"eventId": EventsLocal().get_test_event_id()},
            "body": json.dumps(
                {
                    "location_id": 2,  # TODO: get test id everywhere.
                    "organizers_profile_id": 2,
                    "website_url": "https://example.com",
                    "facebook_event_url": "https://facebook.com/event",
                    "meetup_event_url": "https://meetup.com/event",
                    "registration_url": "https://example.com/register",
                }
            ),
            "headers": create_authorization_http_headers(user_jwt),
        }

        # Call the function
        response = updateHandler(event, None)

        # Assert that the response is correct
        assert response["statusCode"] == HTTPStatus.OK.value
        assert response["body"] == json.dumps({"message": "Event updated successfully"})

        # Ensure error without headers
        event.pop("headers")
        response = updateHandler(event, None)
        assert response["statusCode"] == HTTPStatus.OK.value, response

    @staticmethod
    def test_update_invalid_input():
        # Define a mock event with invalid input data
        invalid_event = {
            "pathParameters": {"eventId": "1"},
            "body": json.dumps(
                {
                    "location_id": 0,
                    "organizers_profile_id": 2,
                    "website_url": "invalid url",
                    "facebook_event_url": "https://facebook.com/event",
                    "meetup_event_url": "https://meetup.com/event",
                    "registration_url": "https://example.com/register",
                }
            ),
        }
        # Call the function
        response = updateHandler(invalid_event, None)

        # Assert that the response is correct
        assert response["statusCode"] != HTTPStatus.OK.value
        assert "error" in response["body"]


if __name__ == "__main__":
    test = TestUpdate()
    test.test_update_valid_input()
    # test.test_update_invalid_input()



from src.events_local import EventsLocal
from src.event import Event
from opensearch_local.our_opensearch import OurOpenSearch


# https://opensearch.play1.circ.zone:5601/app/login?
def test_foreach():

    open_search_ingester = OurOpenSearch(buffer_size=5)

    # where = "is_test_data = 1"
    where = "event_id = 1345"
    limit = 10
    function = open_search_ingester.insert

    events_local = EventsLocal(is_test_data=True)

    index_name = "event"

    # select_function = events_local.get_values_dict_list
    select_function = events_local.get_values_dict_list_by_where

    # responses_dict = events_local.foreach(where, limit, function, id_column_name="event_id", index_name=index_name)
    responses_dict = events_local.foreach(where, limit, function, id_column_name="event_id", select_function=select_function, index_name=index_name)

    print(responses_dict, "\n")

    body = {
        "query": {
            "match": {
                "is_test_data": "true"
            }
        }
    }

    # The qeury for the dev tool in dashboard:
    # GET event/_search
    # {
    # "query": {
    #     "match": {
    #     "is_test_data": "true"
    #     }
    # }
    # }

    search_response = open_search_ingester.search(index=index_name, body=body)

    print(search_response)


def test_to_json_from_db():
    events_local = EventsLocal()

    result = events_local.select_by_event_id(202408182100165)
    # result = events_local.select_one_dict_by_where()

    print(result)

    event_one = Event(**result)

    print(event_one)

    assert event_one.get("event_id") == result["event_id"]

    event_json = event_one.to_json_dict()

    assert event_json["kwargs"]["event_id"] == int(result["event_id"])
    assert event_json["kwargs"]["website_url"] == result["website_url"]
    assert event_json["kwargs"]["facebook_event_url"] == result["facebook_event_url"]
    assert event_json["kwargs"]["meetup_event_url"] == result["meetup_event_url"]


if __name__ == '__main__':
    # test_foreach()
    test_to_json_from_db()
